<?php

namespace xenarabia\REP001\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;


class RepEntity extends Entity{


	public function _postSave()
	{
		$visitor = \XF::visitor();

		// Get reputation point score as per user group
		$getRepPointsScore = $this->finder('XF:UserGroup')->where('user_group_id', $visitor->user_group_id)->fetchOne();

		// Update trophy points with reputation point 
		$UpdateTrophy = \XF::app()->find('XF:User', $this->rep_receiver_id);
		$UpdateTrophy->trophy_points = $UpdateTrophy->trophy_points + $getRepPointsScore->rep_points;
		$UpdateTrophy->save();

		// Send alert to user 
		$getuser = \XF::app()->find('XF:User', $this->rep_receiver_id);
		$threads = \XF::app()->find('XF:Thread', $this->rep_thread_id);
		
		/** @var XF\Repository\UserALert $alertRepo */
		$alertRepo = \XF::app()->repository('XF:UserAlert');
		$extra = [];
		$extra = array_merge([
            'threads'	=> $threads,
			'prefix_id' => $threads->prefix_id,
			'title' => $threads->title,
			'link'	=> $this->app()->router('public')->buildLink('nopath:threads', $threads),
			'depends_on_addon_id' => 'xenarabia/REP001',
		], $extra);
		$alertRepo->alert($getuser, $visitor->user_id, $visitor->username, 'user', $getuser->user_id, 'rep_add', $extra);
	}

    public static function getStructure(Structure $structure)
	{
		$options = \XF::options();
		$structure->table = 'xa_reputation';
		$structure->shortName = 'xenarabia\REP001:RepEntity';
		$structure->primaryKey = 'rep_id';
		$structure->columns = [
			'rep_id' => ['type' => self::UINT, 'autoIncrement' => true],		
			'rep_sender_id' => ['type' => self::UINT ],					
			'rep_receiver_id' => ['type' => self::UINT ],			
			'rep_post_id' => ['type' => self::UINT ],			
			'rep_thread_id' => ['type' => self::UINT ],			
			'rep_text' => ['type' => self::STR, 'maxLength' => $options->rep_max_text, 'required' => 'please_enter_rep_msg'],			
			'rep_date' => ['type' => self::UINT, 'default' => \XF::$time],	
			'rep_rate' => ['type' => self::UINT, 'default' => 0 ],			
		];
		$structure->getters = [];
		$structure->relations = [
			'SenderUser' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => [['user_id', '=', '$rep_sender_id']],
				'primary' => true
			],
			'ReciverUser' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => [['user_id', '=', '$rep_receiver_id']],
				'primary' => true
			],
			'Thread' => [
				'entity' => 'XF:Thread',
				'type' => self::TO_ONE,
				'conditions' => [['thread_id', '=', '$rep_thread_id']],
				'primary' => true
			],
		];
		
		
		return $structure;
    }
}