<?php

namespace xenarabia\REP001\Pub\Controller;

use MathParser\Interpreting\Visitors\Visitor;
use xenarabia\REP001\Entity\RepEntity;
use XF;
use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\Mvc\Reply\View;
use XF\Pub\Controller\AbstractController;


class Reputation extends AbstractController
{


    public function actionIndex(ParameterBag $params)
    {
        return $this->error(\XF::phrase('requested_page_not_found'), 404);       
    }

    public function actionAdd(ParameterBag $params)
    {
        $visitor = \XF::visitor();
        $limitTime = \XF::$time + 86400;
        $getTotalRep = $this->finder('xenarabia\REP001:RepEntity')
        ->where('rep_sender_id', $visitor->user_id)
        ->where('rep_date', '<=',$limitTime)->total();

        // Check if user exceeded daily reputation limit
        if($visitor->hasPermission('xaReputationSys', 'maxReputationPerDay') >= $getTotalRep)
        {
            return $this->error(\XF::phrase('rep_have_exceeded_the_daily_limit'));
        }
        $postId = $this->filter('post_id', 'int');
        $userpost = $this->finder('XF:Post')->with('Thread', true)->where('post_id', $postId)->fetchOne();
        $viewParams = [
            'userpost'    => $userpost
        ];
        return $this->view('xenarabia\REP001\add', 'rep_add', $viewParams);
    }

    public function actionSave(ParameterBag $params)
    {

        $visitor = \XF::visitor();

        $checkUserRep = $this->finder('xenarabia\REP001:RepEntity')
        ->where('rep_sender_id', $visitor->user_id)
        ->where('rep_post_id', $this->filter('rep_post_id', 'int'))->total();
        
        $thread_id = $this->filter('rep_thread_id', 'int');
        $threads = \XF::finder('XF:Thread')->where('thread_id', $thread_id)->fetchOne();

        if($checkUserRep > 0)
        {
            return $this->redirect($this->buildLink('threads', $threads), \XF::phrase('rep_you_already_rated_this_post'));
        }

        $rep                        = $this->em()->create('xenarabia\REP001:RepEntity');
        $rep->rep_sender_id         =    $this->filter('rep_sender_id', 'int');
        $rep->rep_receiver_id       =    $this->filter('rep_receiver_id', 'int');
        $rep->rep_post_id           =    $this->filter('rep_post_id', 'int');
        $rep->rep_thread_id         =    $this->filter('rep_thread_id', 'int');
        $rep->rep_text              =    $this->filter('rep_text', 'str');
        $rep->rep_rate              =    $this->filter('rep_rate', 'int');
        $rep->save();

        return $this->redirect($this->buildLink('threads', $threads), \XF::phrase('rep_has_been_saved'));
    }

    protected function repEditSave(RepEntity $repEntity)
    {
        $entityInput = $this->filter([
            'rep_sender_id' => 'int',
            'rep_receiver_id' => 'int',
            'rep_text' => 'str',
            'rep_rate' => 'int',
            'rep_post_id' => 'int',
            'rep_thread_id' => 'int',
        ]);

        $form = $this->formAction();
        $form->basicEntitySave($repEntity, $entityInput);

        return $form;
    }


    public function actionRemove(ParameterBag $params)
    {
        if ($this->isPost()) {

            $getPostThread = \XF::finder('xenarabia\REP001:RepEntity')->where('rep_id', $params->rep_id)->fetchOne();
            $threads = \XF::finder('XF:Thread')->where('thread_id', $getPostThread->rep_thread_id)->fetchOne();
            \XF::db()->delete('xa_reputation','rep_id = ?', [$params->rep_id]);

            // Update trophy points with reputation point 
            $visitor = \XF::visitor();
            $getRepPointsScore = $this->finder('XF:UserGroup')->where('user_group_id', $visitor->user_group_id)->fetchOne();

            $UpdateTrophy = \XF::app()->find('XF:User', $getPostThread->rep_receiver_id);
            $UpdateTrophy->trophy_points = $UpdateTrophy->trophy_points - $getRepPointsScore->rep_points;
            $UpdateTrophy->save();

            $redirect = $this->getDynamicRedirect(null, false);
            $reply = $this->redirect($redirect);
            $reply->setJsonParam('message', \XF::phrase('rep_has_been_removed'));
            return $reply;
        }
    }


    protected function assertRpExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('xenarabia\REP001:RepEntity', $id, $with, $phraseKey);
    }


}
